import * as React from 'react';
import { mergeCallbacks, usePopoverContext_unstable } from '@fluentui/react-components';
import { Preview, PreviewContent, PreviewHeader, PreviewMetadata, PreviewSurface, PreviewTrigger } from '@fluentui-copilot/react-preview';
import { SensitivityLabel } from '@fluentui-copilot/react-sensitivity-label';
import { Citation } from '../../Citation';
import { Reference } from '../../ReferenceV2';
// Abstract the citation that serves as the close button of the Preview into its own component so that we have access to the popover context.
const CitationCloseButton = citationCloseButtonProps => {
  const setPopoverOpen = usePopoverContext_unstable(context => context.setOpen);
  const onPopoverCloseClick = React.useCallback(ev => {
    setPopoverOpen(ev, false);
    // Don't navigate
    ev.preventDefault();
  }, [setPopoverOpen]);
  const onCitationCloseButtonClick = mergeCallbacks(onPopoverCloseClick, citationCloseButtonProps.onClick);
  return /*#__PURE__*/React.createElement(Citation, {
    ...citationCloseButtonProps,
    onClick: onCitationCloseButtonClick
  });
};
/**
 * Function to generate a Citation - Reference pair with correctly wired props.
 * Both the Citation and the Reference act as trigger components to Previews that are also correctly wired by the function.
 * Props can either be passed in to the function or applied directly to the generated components in JSX.
 *
 * A `useReferenceCitationPreview` hook exists that calls this function and memoizes its return value based on the props.
 * You should use the hook when the props used to generate the Citation - Reference pair can change dynamically.
 * If they are static, or in case you wanted to generate multiple pairs within a cycle, you should call
 * `generateReferenceCitationPreview` directly.
 *
 * @returns A Citation - Reference pair with correctly wired props.
 */
export const generateReferenceCitationPreview = props => {
  const {
    citationPreviewProps,
    citationPreviewSurfaceProps,
    citationProps,
    index,
    previewContentProps,
    previewHeaderProps,
    referencePreviewProps,
    referencePreviewSurfaceProps,
    referenceProps,
    sensitivityLabelProps
  } = props;
  const CitationWithPreview = props => {
    var _previewContentProps_content;
    return /*#__PURE__*/React.createElement(Preview, {
      closeOnScroll: true,
      positioning: "above",
      size: "small",
      trapFocus: true,
      withArrow: true,
      ...citationPreviewProps
    }, /*#__PURE__*/React.createElement(PreviewTrigger, null, /*#__PURE__*/React.createElement(Citation, {
      ...citationProps,
      ...props
    }, index)), /*#__PURE__*/React.createElement(PreviewSurface, citationPreviewSurfaceProps, /*#__PURE__*/React.createElement(PreviewContent, null, /*#__PURE__*/React.createElement(PreviewHeader, {
      citation: /*#__PURE__*/React.createElement(CitationCloseButton, {
        "aria-label": `Close Reference ${index}`,
        referenceHref: "#",
        tabIndex: -1
      }, index),
      media: referenceProps === null || referenceProps === void 0 ? void 0 : referenceProps.graphic
    }, referenceProps === null || referenceProps === void 0 ? void 0 : referenceProps.children), /*#__PURE__*/React.createElement(PreviewMetadata, {
      icon: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.icon,
      primaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.primaryText,
      secondaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.secondaryText,
      tertiaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.tertiaryText
    }), sensitivityLabelProps && /*#__PURE__*/React.createElement(SensitivityLabel, sensitivityLabelProps), (_previewContentProps_content = previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.content) !== null && _previewContentProps_content !== void 0 ? _previewContentProps_content : citationPreviewProps === null || citationPreviewProps === void 0 ? void 0 : citationPreviewProps.children)));
  };
  const ReferenceWithPreview = props => {
    const mergedReferenceProps = {
      citation: index,
      ...referenceProps,
      ...props
    };
    var _previewHeaderProps_children, _previewContentProps_content;
    return /*#__PURE__*/React.createElement(Preview, {
      closeOnScroll: true,
      positioning: "above",
      size: "small",
      trapFocus: true,
      withArrow: true,
      ...referencePreviewProps
    }, /*#__PURE__*/React.createElement(PreviewTrigger, null, /*#__PURE__*/React.createElement(Reference, mergedReferenceProps)), /*#__PURE__*/React.createElement(PreviewSurface, referencePreviewSurfaceProps, /*#__PURE__*/React.createElement(PreviewContent, null, /*#__PURE__*/React.createElement(PreviewHeader, {
      citation: /*#__PURE__*/React.createElement(CitationCloseButton, {
        "aria-label": `Close Reference ${index}`,
        referenceHref: "#",
        tabIndex: -1
      }, index),
      media: referenceProps === null || referenceProps === void 0 ? void 0 : referenceProps.graphic,
      ...previewHeaderProps
    }, (_previewHeaderProps_children = previewHeaderProps === null || previewHeaderProps === void 0 ? void 0 : previewHeaderProps.children) !== null && _previewHeaderProps_children !== void 0 ? _previewHeaderProps_children : referenceProps === null || referenceProps === void 0 ? void 0 : referenceProps.children), /*#__PURE__*/React.createElement(PreviewMetadata, {
      icon: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.icon,
      primaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.primaryText,
      secondaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.secondaryText,
      tertiaryText: previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.tertiaryText
    }), sensitivityLabelProps && /*#__PURE__*/React.createElement(SensitivityLabel, sensitivityLabelProps), (_previewContentProps_content = previewContentProps === null || previewContentProps === void 0 ? void 0 : previewContentProps.content) !== null && _previewContentProps_content !== void 0 ? _previewContentProps_content : referencePreviewProps === null || referencePreviewProps === void 0 ? void 0 : referencePreviewProps.children)));
  };
  const components = {
    Citation: CitationWithPreview,
    Reference: ReferenceWithPreview
  };
  return components;
};
//# sourceMappingURL=generateReferenceCitationPreview.js.map